#! /bin/sh
# ======================================================================
# This script creates a set of useful links to the zchaff libraries
# so you can use default directory paths when configuring NuSMV
# ======================================================================

# ----------------------------------------------------------------------

. zchaff-default

ZCHAFF_TARGETS="libsat.a SAT_C.h"

# ----------------------------------------------------------------------
# Inner helpers:
LN="ln -s"
DELETE="rm -f"
# ----------------------------------------------------------------------

MAKE="make"
if [ `uname -s` = SunOS ]; then
MAKE="gnumake"
fi

if [ ! -d $zchaff_topdir ]; then 
 echo "Untarring the archive <$zchaff_archive>..."
 type=`file $zchaff_archive | cut -d \  -f 2`

 if [ "${type}x" = "Zipx" ]; then
  unzip $zchaff_archive
 elif [ "${type}x" = "gzipx" ]; then  
  gzip -cd $zchaff_archive | tar xf -
 elif [ "${type}x" = "bzip2x" ]; then  
  bzip2 -cd $zchaff_archive | tar xf -
 else
  echo "Error: do not know how to decompress the archive $zchaff_archive"
  echo "Decompress it manually, then re-run build.sh"
 fi 
fi

if [ ! -d $zchaff_topdir ]; then 
echo "<$zchaff_topdir> directory was aspected to exist after unpacking $zchaff_archive"
exit 1
fi

# applies the patch if needed
if [ ! -f patched ]; then
  patch -p0 -N < $zchaff_patch && touch patched
fi



cd $zchaff_topdir
echo "Changed directory to <`pwd`>"


# ----------------------------------------------------------------------
# Make invokation:
echo
echo "Building zchaff using <$MAKE $*>..."
$MAKE $* $ZCHAFF_TARGETS
echo
# ----------------------------------------------------------------------

cd ..
echo "Changed directory to <`pwd`>"

lib_name="libsat.a"
if [ ! -r $zchaff_topdir/$lib_name ]; then 
echo "Library $zchaff_topdir/$lib_name has not been successfully built. Abort."
exit 1
fi

# link
if [ ! -h $zchaff_link ]; then
  $LN ./zchaff/$zchaff_topdir $zchaff_link
fi

echo "Done."
